/*
 ==============================================================================================================================================
 Author      : CHEONG SE HANG (dit.dhc@lostcity-studio.com)
 Description : TCP client.
 ==============================================================================================================================================
 */
#ifndef CLIENT_H_
#define CLIENT_H_
#include <string.h>
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                                  Constants
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
#define CONNECT_TIMEOUT 500     /* The timeout on connect(). In million seconds. */
#define GROUP_MAX_NUM 16            /* Max number of elevators supported in the system. */
#define SEND_QUEUE 128          /* The size of sending queue. */
#define IP_ADJUST -251          /* A number which match a highest position of IPv4 to the array peer list. */
#define TCP_MAXLINE 4096
#define TCP_PORT 23456




/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Structure Definitions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
struct SendDataStruct { /* The structure of data. */
    char send_buf[ TCP_MAXLINE ];         /* Send buffer. */
    void* handler;          /* The hander which handles the data received. */
};
//struct SendDataStruct_Ptr { /* The structure of data. */
//    char* send_buf;         /* Send buffer. */
//    void* handler;          /* The hander which handles the data received. */
//};




/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
/**
 * Close an TCP connection on client.
 * Note: This function doesn't affect any info in @PeerList.
 * @param scokfd            Socket descriptor.
 */
void Client_CloseConnection ( int sockfd );


/**
 * Construct a TCP client. It initiates any lock or queue of a TCP client.
 * Note: DON'T RECALL IT TWICE BEFORE Client_Destructor() IS CALLED. Otherwise, It may deal with memory leak problem.
 * @param IPAddr            IP Address in string format (a.b.c.d).
 */
void Client_Constructor ( const char* IPAddr );


/**
 * Free a TCP client with its resources.
 * Note: DON'T CALL IT TWICE BEFORE Client_Constructor(). Otherwise, segmentation fault.
 * @param IPAddr            IP Address in string format (a.b.c.d).
 */
void Client_Destructor ( const char* IPAddr );


/**
 * Pend a message to a peer. This function can't guarantee whether the message is sent or not.
 * If the sending buffer is full or remote server is off during transmission, the message may lost and without any notification.
 * @param IPAddr            IP Address in string format (a.b.c.d).
 * @param data              Data information. Refer to @SendDataStruct.
 */
void Client_Pend ( const char* IPAddr, struct SendDataStruct *data );
//void Client_Pend ( const char* IPAddr, char *msg );


/**
 * Initiate the static variables in the TCP client service.
 */
void Client_Static_Initialization ();


/**
 * Test to connect a TCP server.
 * Note: This function doesn't affect any info in @PeerList. It is safe to test whether a TCP server is still available.
 *       DON'T forget to call Client_CloseConnection() and close the connection.
 * @param scokfd            Socket descriptor.
 * @param IPAddr            IP Address in string format (a.b.c.d).
 * @return Returns 1 on success. Otherwise, returns 0.
 */
int Client_TestConnection ( int* sockfd, const char* IPAddr );


/**
 * Get the corresponding position in @PeerList by a string of IP address.
 * Note: If @IPAddr is incorrect, the returned result is rubbish or segmentation fault.
 * @param IPAddr            String of IP address.
 * @return Returns the index in @PeerList.
 */
int Client_GetPosition ( const char* IPAddr );


//xxx [DEBUG]
void Client_Debugger ();
void Test();




#endif /* CLIENT_H_ */
