/*
 ==============================================================================================================================================
 Author      : CHEONG SE HANG (dit.dhc@lostcity-studio.com)
 Description : Extension to pthread library.
 ==============================================================================================================================================
 */
#include "Log.h"
#include "Pthread.h"
#include <stdio.h>
#include <string.h>
/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Private Variables
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
#ifdef LOGGING_ENABLE
static char log[ 128 ];                 /* Buffer for logging. */
#endif




/*---------------------------------------------------------------------------------------------------------------------------------------------
 *
 *                                                              Public Functions
 *
 *-------------------------------------------------------------------------------------------------------------------------------------------*/
int Pthread_attr_init ( pthread_attr_t* attr, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_attr_init( attr ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_attr_init() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_attr_setdetachstate ( pthread_attr_t* attr, int detachstate, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_attr_setdetachstate( attr, detachstate ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_attr_setdetachstate() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_cond_destroy ( pthread_cond_t* cond, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_cond_destroy( cond ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_cond_destroy() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_cond_init ( pthread_cond_t* cond, const pthread_condattr_t* attr, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_cond_init( cond, attr ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_cond_init() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_cond_signal ( pthread_cond_t* cond, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_cond_signal( cond ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_cond_signal() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


//int Pthread_cond_timedwait ( pthread_cond_t* cond, pthread_mutex_t* mutex, const struct timespec* abstime ) {
//    int s;
//    if ( ( s = pthread_cond_timedwait( cond, mutex, abstime ) ) != 0 ) {
//        Err2Log( PAGE_NO_2002, s, PTHREAD_COND_TIMEDWAIT_, __FILE__, __LINE__, 1 );
//        return -1;
//    }
//    return 0;
//}


int Pthread_cond_wait ( pthread_cond_t* cond, pthread_mutex_t* mutex, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_cond_wait( cond, mutex ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_cond_wait() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_create ( pthread_t* thread, const pthread_attr_t* attr, void*(*start_routine)(void*), void* arg,
    const char* filename, int line ) {
    int s;
    if ( ( s = pthread_create( thread, attr, start_routine, arg ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_create() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


//int Pthread_detach ( pthread_t thread ) {
//    int s;
//    if ( ( s = pthread_detach( thread ) ) != 0 ) {
//        Err2Log( PAGE_NO_2002, s, PTHREAD_DETACH_, __FILE__, __LINE__, 1 );
//        return -1;
//    }
//    return 0;
//}
//
//
//int Pthread_join ( pthread_t thread, void** value_ptr ) {
//    int s;
//    if ( ( s = pthread_join( thread, value_ptr ) ) != 0 ) {
//        Err2Log( PAGE_NO_2002, s, PTHREAD_JOIN_, __FILE__, __LINE__, 1 );
//        return -1;
//    }
//    return 0;
//}


int Pthread_mutex_destroy ( pthread_mutex_t* mutex, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_mutex_destroy( mutex ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_mutex_destroy() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_mutex_init ( pthread_mutex_t* mutex, const pthread_mutexattr_t* attr, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_mutex_init( mutex, attr ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_mutex_init() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_mutex_lock ( pthread_mutex_t* mutex, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_mutex_lock( mutex ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_mutex_lock() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_mutex_unlock ( pthread_mutex_t* mutex, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_mutex_unlock( mutex ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_mutex_unlock() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlock_destroy ( pthread_rwlock_t* rwlock, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlock_destroy( rwlock ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlock_destroy() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlock_init ( pthread_rwlock_t* rwlock, const pthread_rwlockattr_t* attr, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlock_init( rwlock, attr ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlock_init() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlock_rdlock ( pthread_rwlock_t* rwlock, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlock_rdlock( rwlock ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlock_rdlock() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlock_unlock ( pthread_rwlock_t* rwlock, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlock_unlock( rwlock ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlock_unlock() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlock_wrlock ( pthread_rwlock_t* rwlock, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlock_wrlock( rwlock ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlock_wrlock() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlockattr_destroy ( pthread_rwlockattr_t* attr, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlockattr_destroy( attr ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlockattr_destroy() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlockattr_init ( pthread_rwlockattr_t* attr, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlockattr_init( attr ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlockattr_init() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlockattr_setkind_np ( pthread_rwlockattr_t* attr, int pref, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlockattr_setkind_np( attr, pref ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlockattr_setkind_np() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}


int Pthread_rwlockattr_setpshared ( pthread_rwlockattr_t *attr, int pshared, const char* filename, int line ) {
    int s;
    if ( ( s = pthread_rwlockattr_setpshared( attr, pshared ) ) != 0 ) {
#ifdef LOGGING_ENABLE
        sprintf( log, "%s(%d) -> pthread_rwlockattr_setpshared() failed: %s", filename, line, strerror( s ) );
        Log( log );
#endif
        return -1;
    }
    return 0;
}
